﻿/*
 * Created by SharpDevelop.
 * User: pshai
 * Date: 27.9.2017
 * Time: 17:57
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TradeServer.ScriptingDriver.DataObjects;
using TradeServer.ScriptingDriver.Interfaces;
using TicTacTec.TA.Library;



namespace ScriptIndicators
{
    public class TMF : IIndicatorScript
    {
        int length = 21;

        double value;

        double lastWMA = 0, lastWMV = 0;

        #region IIndicatorScript implementation
        public void Init()
        {

        }

        public IndicatorResult GetResult(IndicatorContext context)
        {
            try
            {
                IPriceInstrument instrument = context.PriceInstrument;
                int position = instrument.Close.Length - (1 + context.Offset);
                var high = instrument.High[position];
                var low = instrument.Low[position];
                var close = instrument.Close[position];
                var prevClose = instrument.Close[position - 1];
                var volume = instrument.Volume[position];
                var prevVolume = instrument.Volume[position - 1];
                
                var tr_h = Math.Max(prevClose, high);
                var tr_l = Math.Min(prevClose, low);
                var tr_c = tr_h - tr_l;
                var adv = volume * (((close - tr_l) - (tr_h - close)) / (tr_c > 0 ? tr_c : 9999999));
                var wv = volume + prevVolume;
                var wmV = wima(wv, length, lastWMV);
                var wmA = wima(adv, length, lastWMA);

                lastWMV = wmV;
                lastWMA = wmA;
                
                // TMF
                value = wmV > 0 ? wmA / wmV : 0;

                /*
                tr_h=max(close[1],high)
                tr_l=min(close[1],low)
                tr_c=tr_h-tr_l
                adv=volume*((close-tr_l)-(tr_h-close))/ iff(tr_c==0,9999999,tr_c)
                wv=volume+(volume[1]*0)
                wmV= WiMA(wv,length)
                wmA= WiMA(adv,length)
                tmf= iff(wmV==0,0,wmA/wmV)
                plot(tmf, style=area)
                */

                /*
                context.Logger.LogToFile(string.Format("tr_h= {0}\n\rtr_l= {1}\n\rtr_c= {2}\n\radv= {3}\n\rwv= {4}\n\rwmV= {5}\n\rwmA= {6}\n\rVALUE= {7}\n\r-----------",
                    tr_h,
                    tr_l,
                    tr_c,
                    adv,
                    wv,
                    wmV,
                    wmA,
                    value
                    ));
                    */

                if (value > 0)
                    return IndicatorResult.BuyLong;
                else
                    return IndicatorResult.SellShort;
            }
            catch (Exception ex) // This try catch block is an highly recommend practice
            {
                context.Logger.LogToFile(string.Format("Exception detected: {0}", ex));
            }

            return IndicatorResult.StayNoPosition; // Default output
        }

        
        double wima(double src, int len, double last)
        {
            var temp = last * (len - 1);
            if (double.IsNaN(temp)) temp = 0;
            var rtn = (src + temp) / len;
            return rtn;
        }

        public List<ScriptParameter> GetParameters()
        {
            var rtn = new List<ScriptParameter>
            {
                new ScriptParameter("Length", ScriptParameterType.Integer, length.ToString())
            };

            return rtn;
        }

        public void SetParameters(Dictionary<string, object> parameters)
        {
            length = ParseInt(parameters["Length"], length);
        }

        public List<ChartRecord> GetChartLines()
        {
            List<ChartRecord> chartLines = new List<ChartRecord>
            {
                new ChartRecord() { ChartIndex = 1, Name = "0", HexLineColor = "#888888" },
                new ChartRecord() { ChartIndex = 1, Name = "TMF", HexLineColor = "#FF0000" }
            };

            return chartLines;
        }

        public List<decimal> GetChartData()
        {
            var chartData = new List<decimal>
            {
                0m,
                Convert.ToDecimal(value)
            };

            return chartData;
        }

        public string Name
        {
            get
            {
                return "pshai's TMF";
            }
        }

        public int TicksBack
        {
            get
            {
                return length;
            }
        }
        #endregion

        int ParseInt(object value, int lastValue)
        {
            try
            {
                return Convert.ToInt32(value.ToString());
            }
            catch
            {
                return lastValue;
            }
        }
    }
}
